#include <msp/core/maputils.h>
#include "tool.h"
#include "toolchain.h"

using namespace std;
using namespace Msp;

Toolchain::~Toolchain()
{
	for(ToolMap::iterator i=tools.begin(); i!=tools.end(); ++i)
		delete i->second;
	for(ToolchainList::iterator i=chains.begin(); i!=chains.end(); ++i)
		delete *i;
}

void Toolchain::add_tool(Tool *tool)
{
	insert_unique(tools, tool->get_tag(), tool);
}

void Toolchain::add_toolchain(Toolchain *chain)
{
	chains.push_back(chain);
}

bool Toolchain::has_tool(const string &tag) const
{
	if(tools.count(tag))
		return true;
	for(ToolchainList::const_iterator i=chains.begin(); i!=chains.end(); ++i)
		if((*i)->has_tool(tag))
			return true;
	return false;
}

Tool &Toolchain::get_tool(const string &tag) const
{
	if(!tools.count(tag))
	{
		for(ToolchainList::const_iterator i=chains.begin(); i!=chains.end(); ++i)
			if((*i)->has_tool(tag))
				return (*i)->get_tool(tag);
	}

	return *get_item(tools, tag);
}

Tool *Toolchain::get_tool_for_suffix(const string &suffix, bool aux) const
{
	for(ToolMap::const_iterator i=tools.begin(); i!=tools.end(); ++i)
		if(i->second->accepts_suffix(suffix, aux))
			return i->second;

	for(ToolchainList::const_iterator i=chains.begin(); i!=chains.end(); ++i)
		if(Tool *tool = (*i)->get_tool_for_suffix(suffix, aux))
			return tool;

	return 0;
}
